﻿Imports System.Runtime.InteropServices
Imports System.Text

Public Class bp8661


	''' <summary>
	''' Sollte eine Funktion als Resultat einen Fehler ausgeben (kleiner 0) kann mit dieser Funktion eine passender Fehlertext ausgegeben werden. 
	''' </summary>
	''' <param name="res"></param>
	''' <returns></returns>
	Public Shared Function GetError(ByVal res As Integer) As Exception

		'Kein Fehler wenn das Resultat einer Funktion 0 oder grösser ist.
		If res >= 0 Then Return Nothing

		Select Case res
			Case -1
				Return New Exception("Unknown Error or COM-Port Is out of range")
			Case -120
				Return New Exception("Error occured while writing to COM port")
			Case -121
				Return New Exception("Error occured while reading from COM port")
			Case -122
				Return New Exception("No response from device (timeout)")
			Case -123
				Return New Exception("Write complement error")
			Case -124
				Return New Exception("Error while receiving characters from port")
			Case -130
				Return New Exception("Invalid COM port (1-based)")
			Case -131
				Return New Exception("COM port could Not be opened")
			Case -132
				Return New Exception("Could Not retrieve COM port settings")
			Case -133
				Return New Exception("Could Not adjust COM port settings")
			Case -134
				Return New Exception("Device sends NAK Not Acknowlege")
			Case -135
				Return New Exception("Device sends wrong data")
			Case Else
				Return New Exception("Unknown Error")
		End Select

	End Function


	''' <summary>
	''' Open serial interface and return the port.
	''' </summary>
	''' <param name="comPort">Specifies the COM-Port to be used to communicate with the instrument.1 - COM1, 2 - COM2, ...65</param>
	''' <returns>=>0 ein gültiges Händel, kleiner Null eine Fehlernummer</returns>
	<DllImport("bp8661.dll", CallingConvention:=CallingConvention.Cdecl)>
	Public Shared Function bp8661_OpenInterface(ByVal comPort As Integer) As Integer
	End Function

	''' <summary>
	''' Close an opened interface
	''' </summary>
	''' <param name="portHandle">Handle value which was returned from initialize Function OpenInterface() To have access To the assigned instrument.</param>
	''' <returns>1=Success, -1=Error</returns>
	<DllImport("bp8661.dll", CallingConvention:=CallingConvention.Cdecl)>
	Public Shared Function bp8661_CloseInterface(ByVal portHandle) As Integer
	End Function

	''' <summary>
	''' Set the interface settings.
	''' </summary>
	''' <param name="baudrate">Baudrate of the Interface. Valid Range: 115200</param>
	''' <param name="databits">Databits of the Interface. Valid Range: 8</param>
	''' <param name="parity">Parity of the Interface. Valid Range: 0</param>
	''' <param name="stopbits">Stopbits of the Interface. Valid Range: 1</param>
	''' <returns>1=Success, less then 0=Error</returns>
	<DllImport("bp8661.dll", CallingConvention:=CallingConvention.Cdecl)>
	Public Shared Function bp8661_SetInterfaceConfiguration(ByVal baudrate As Integer, ByVal databits As Integer, ByVal parity As Integer, ByVal stopbits As Integer) As Integer
	End Function

	''' <summary>
	''' Setzt die Einstellungen des Sensors. Da der 8661 nur ein Set an Werten zulässt, müssen diese nicht eingegeben werden.
	''' </summary>
	''' <param name="Err">Falls ein Fehler auftritt wird dieser zurückgegeben.</param>
	''' <returns></returns>
	Public Shared Function bp8661_SetInterfaceConfiguration(Optional Err As Exception = Nothing) As Integer
		Dim Res As Integer = bp8661_SetInterfaceConfiguration(115200, 8, 0, 1)

		Err = GetError(Res)
		Return Res
	End Function

	''' <summary>
	''' Sets the timeout in milliseconds for the serial communication With the device
	''' </summary>
	''' <param name="timeoutInMs">Timeout value for timeout in milliseconds. Value Range: 1-100'000</param>
	''' <returns>1=Success, -1=Error</returns>
	<DllImport("bp8661.dll", CallingConvention:=CallingConvention.Cdecl)>
	Public Shared Function bp8661_SetResponseTimeoutMs(ByVal timeoutInMs As Integer) As Integer
	End Function

	''' <summary>
	''' Sets the buffer length for receive and send buffer.
	''' </summary>
	''' <param name="portHandle">Handle value which was returned from initialize Function OpenInterface() to have access To the assigned instrument.</param>
	''' <param name="inputlength">Serial interface input buffer length as bytes. Value Range: 1-128000</param>
	''' <param name="outputlength">Serial interface output buffer length as bytes. Value Range: 1-128000</param>
	''' <returns>1=success, less the 0=error</returns>
	<DllImport("bp8661.dll", CallingConvention:=CallingConvention.Cdecl)>
	Public Shared Function bp8661_SetPortQueueLen(ByVal portHandle As Integer, ByVal inputlength As Integer, ByVal outputlength As Integer) As Integer
	End Function

	''' <summary>
	''' Gets the version of the dll
	''' </summary>
	''' <param name="libVersion">The version of the dll</param>
	<DllImport("bp8661.dll", CallingConvention:=CallingConvention.Cdecl, CharSet:=CharSet.Auto, ExactSpelling:=True)>
	Public Shared Sub bp8661_GetLibVersion(<MarshalAs(UnmanagedType.LPStr)> libVersion As System.Text.StringBuilder)
	End Sub

	''' <summary>
	''' Gibt die Version der DLL zurück.
	''' </summary>
	''' <returns></returns>
	Public Shared Function bp8661_GetLibVersion() As String
		Dim Res As New StringBuilder
		bp8661_GetLibVersion(Res)
		Return Res.ToString
	End Function

	''' <summary>
	''' Gets the creation date of the dll
	''' </summary>
	''' <param name="libCreationDate"></param>
	<DllImport("bp8661.dll", CallingConvention:=CallingConvention.Cdecl, CharSet:=CharSet.Ansi, ExactSpelling:=True)>
	Public Shared Sub bp8661_GetLibCreationDate(<MarshalAs(UnmanagedType.LPStr)> libCreationDate As System.Text.StringBuilder)
	End Sub

	''' <summary>
	''' Erstelldatum des DLL
	''' </summary>
	''' <returns></returns>
	Public Shared Function bp8661_GetLibCreationDate() As String
		Dim Res As New StringBuilder
		bp8661_GetLibCreationDate(Res)
		Return Res.ToString
	End Function

	''' <summary>
	''' Returns the encoder resolution of the device.
	''' </summary>
	''' <param name="portHandle">Handle value which was returned from initialize Function OpenInterface() To have access To the assigned instrument.</param>
	''' <param name="encoderResolution">Encoder resolution of the device.</param>
	''' <returns></returns>
	<DllImport("bp8661.dll", CallingConvention:=CallingConvention.Cdecl)>
	Public Shared Function bp8661_GetEncoderResolution(ByVal portHandle As Integer, ByRef encoderResolution As Int32) As Integer
	End Function


	<DllImport("bp8661.dll", CallingConvention:=CallingConvention.Cdecl)>
	Public Shared Function bp8661_GetValues(ByVal portHandle As Integer, ByRef valueTorque As Single, ByRef valueSpeed As Single, ByRef valueAngle As Single, ByRef valuePower As Single, ByRef valueIncrement As Single) As Integer
	End Function

	'<DllImport("bp8661.dll", CallingConvention:=CallingConvention.Cdecl)>
	'Public Shared Function bp8661_GetSplayValue(ByVal portHandle As Integer, ByRef splayValue As Single) As Integer
	'End Function


	<DllImport("bp8661.dll", CallingConvention:=CallingConvention.Cdecl)>
	Public Shared Function bp8661_GetSplayValue(ByVal portHandle As Integer, <MarshalAs(UnmanagedType.R4)> splayValue As Single) As Integer
	End Function









	<DllImport("bp8661.dll", CallingConvention:=CallingConvention.Cdecl, CharSet:=CharSet.Auto, ExactSpelling:=True)>
	Public Shared Sub bp8661_GetLibVersion(ByRef libVersion As IntPtr)
	End Sub



	<DllImport("bp8661.dll", CallingConvention:=CallingConvention.Cdecl, CharSet:=CharSet.Auto, ExactSpelling:=True)>
	Public Shared Function bp8661_GetSerialNr(ByVal portHandle As Integer, <Out, MarshalAs(UnmanagedType.LPStr)> libVersion As System.Text.StringBuilder) As Integer
	End Function


	'<DllImport("bp8661.dll", CallingConvention:=CallingConvention.Cdecl, CharSet:=CharSet.Ansi, ExactSpelling:=True)>
	'Public Shared Function bp8661_GetSerialNr(ByVal portHandle As Integer, ByVal serialNumber As IntPtr) As Integer
	'End Function

	Public Shared Function bp8661_GetSerialNr(ByVal portHandle As Integer) As String

		Stop

		Dim strSerialNumber = New StringBuilder(21)
		bp8661_GetSerialNr(1, strSerialNumber)





		Dim p1 As IntPtr = Marshal.AllocHGlobal(Marshal.SizeOf(20))
		bp8661_GetLibVersion(p1)

		Dim p0 As IntPtr
		p0 = Marshal.ReadIntPtr(p1)
		Dim tStr As String = Marshal.PtrToStringAnsi(p0)



		'Dim serialNumberPointer As IntPtr = Marshal.AllocHGlobal(Marshal.SizeOf(IntPtr.Zero))
		Dim serialNumberPointer As IntPtr = Marshal.AllocHGlobal(Marshal.SizeOf(IntPtr.Zero))
		'bp8661_GetSerialNr(portHandle, serialNumberPointer)

		'Dim p0 As IntPtr
		p0 = Marshal.ReadIntPtr(serialNumberPointer)
		tStr = Marshal.PtrToStringAnsi(p0)

		Marshal.FreeHGlobal(p0)
		Return tStr

	End Function


End Class
