' C# Libad Interface
'
' (c) 2015 BMC Messsysteme GmbH
' 
'

Imports System
Imports System.Runtime.InteropServices

Namespace LIBAD4

    Public Class LIBAD
        Public Const AD_CHA_TYPE_ANALOG_IN As Integer = &H1000000
        Public Const AD_CHA_TYPE_ANALOG_OUT As Integer = &H2000000
        Public Const AD_CHA_TYPE_DIGITAL_IO As Integer = &H3000000
        Public Const AD_CHA_TYPE_SYNC As Integer = &H5000000
        Public Const AD_CHA_TYPE_ROUTE As Integer = &H6000000
        Public Const AD_CHA_TYPE_CAN As Integer = &H7000000
        Public Const AD_CHA_TYPE_COUNTER As Integer = &H8000000
        Public Const AD_CHA_TYPE_ANALOG_COUNTER As Integer = &H9000000

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_open(ByVal name As String) As Integer
        End Function

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_close(adh As Integer) As Integer
        End Function

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_discrete_in(adh As Integer, cha As Integer, range As Integer, ByRef data As UInteger) As Integer
        End Function

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_discrete_out(adh As Integer, cha As Integer, range As Integer, data As UInteger) As Integer
        End Function

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_get_line_direction(adh As Integer, cha As Integer, ByRef mask As UInteger) As Integer
        End Function

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_set_line_direction(adh As Integer, cha As Integer, mask As UInteger) As Integer
        End Function

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_sample_to_float(adh As Integer, cha As Integer, range As Integer, data As UInteger, ByRef dbl As Single) As Integer
        End Function

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_sample_to_float64(adh As Integer, cha As Integer, range As Long, data As ULong, ByRef dbl As Double)
        End Function

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_float_to_sample(adh As Integer, cha As Integer, range As Integer, dbl As Single, ByRef data As UInteger) As Integer
        End Function

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_float_to_sample64(adh As Integer, cha As Integer, range As Long, dbl As Double, ByRef data As ULong)
        End Function

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_analog_in(adh As Integer, cha As Integer, range As Integer, ByRef data As Single) As Integer
        End Function

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_analog_out(adh As Integer, cha As Integer, range As Integer, data As Single) As Integer
        End Function

        ' what to store
        Public Const AD_STORE_DISCRETE As Integer = &H1
        Public Const AD_STORE_AVERAGE As Integer = &H2
        Public Const AD_STORE_MIN As Integer = &H4
        Public Const AD_STORE_MAX As Integer = &H8
        Public Const AD_STORE_RMS As Integer = &H10

        ' trigger mode
        Public Const AD_TRG_NONE As Integer = &H0
        Public Const AD_TRG_POSITIVE As Integer = &H1
        Public Const AD_TRG_NEGATIVE As Integer = &H2
        Public Const AD_TRG_INSIDE As Integer = &H3
        Public Const AD_TRG_OUTSIDE As Integer = &H4
        Public Const AD_TRG_DIGITAL As Integer = &H80
        Public Const AD_TRG_NEVER As Integer = &HFF

        ' scan flags
        Public Const AD_SF_SCANNING As Integer = &H1   ' Is set while device is scanning.
        Public Const AD_SF_TRIGGER As Integer = &H2   ' Scan has triggered.
        Public Const AD_SF_SAMPLES As Integer = &H10000   ' Scan buffers handled internally.



        ' Scan channel descriptor.
        ' Defines scan settings of a single channel.
        <StructLayout(LayoutKind.Sequential, Pack:=0)>
        Public Structure ad_scan_cha_desc
            Public cha As Integer                 ' channel type and id
            Public range As Integer               ' range number (driver specific)
            Public store As Integer               ' what to sample
            Public ratio As Integer               ' scan ratio
            Public zero As UInteger               ' physical 0.0
            Public trg_mode As Byte              ' trigger mode
            Public alarm_mode As Byte             ' alarm mode
            Private sc_res1 As Byte
            Private sc_res2 As Byte
            Public trg_par1 As UInteger           ' trigger parameters
            Public trg_par2 As UInteger
            Public samples_per_run As Integer     ' number of samples per run
            Public alarm_par1 As UInteger         ' alarm parameters
            Public alarm_par2 As UInteger
            Private sc_res3 As UInteger
            Private sc_res4 As UInteger
            Private sc_res5 As UInteger
            Private sc_res6 As UInteger
            Private sc_res7 As UInteger
        End Structure

        ' Scan descriptor.
        ' Defines global scan settings.
        <StructLayout(LayoutKind.Sequential, Pack:=0)>
        Public Structure ad_scan_desc
            Public sample_rate As Double          ' sampling rate (sec)
            Public clock_rate As Double           ' ref clock rate (sec)
            Public prehist As ULong               ' number of samples prehistory
            Public posthist As ULong              ' number of samples posthistory
            Public ticks_per_run As UInteger     ' number of ticks per run
            Public bytes_per_run As UInteger      ' bytes per run
            Public samples_per_run As UInteger    ' samples per run
            Public flags As UInteger              ' scan flags 
            Private sd_res1 As UInteger
            Private sd_res2 As UInteger
            Private sd_res3 As UInteger
            Private sd_res4 As UInteger
            Private sd_res5 As UInteger
            Private sd_res6 As UInteger
            Private sd_res7 As UInteger
            Private sd_res8 As UInteger
            Private sd_res9 As UInteger
            Private sd_res10 As UInteger
            Private sd_res11 As UInteger
            Private sd_res12 As UInteger
        End Structure

        ' Current scan state.
        <StructLayout(LayoutKind.Sequential, Pack:=0)>
        Public Structure ad_scan_state
            Public flags As Integer
            Public runs_pending As Integer
            Public posthist As Long
        End Structure

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_calc_run_size(adh As Integer, ByRef scan_desc As ad_scan_desc, chac As UInteger, <[In](), Out(), MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=2)> chav() As ad_scan_cha_desc) As Integer
        End Function

        Public Shared Function ad_calc_run_size(adh As Integer, ByRef scan_desc As ad_scan_desc, ByRef chav() As ad_scan_cha_desc) As Integer
            Return ad_calc_run_size(adh, scan_desc, chav.Length, chav)
        End Function

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_start_scan(adh As Integer, ByRef scan_desc As ad_scan_desc, chac As UInteger, <[In](), Out(), MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=2)> chav() As ad_scan_cha_desc) As Integer
        End Function

        Public Shared Function ad_start_scan(adh As Integer, ByRef scan_desc As ad_scan_desc, ByRef chav() As ad_scan_cha_desc) As Integer
            Return ad_start_scan(adh, scan_desc, chav.Length, chav)
        End Function

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_stop_scan(adh As Integer, ByRef result As Integer) As Integer
        End Function

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_get_next_run_f(adh As Integer, ByRef state As ad_scan_state, ByRef run As UInteger, <Out(), MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=4)> samplev() As Single, samplec As UInteger) As Integer
        End Function

        Public Shared Function ad_get_next_run_f(adh As Integer, ByRef state As ad_scan_state, ByRef run As UInteger, ByRef samplev() As Single) As Integer
            Return ad_get_next_run_f(adh, state, run, samplev, samplev.Length)
        End Function

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_get_next_run_f64(adh As Integer, ByRef state As ad_scan_state, ByRef run As UInteger, <Out(), MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=4)> samplev() As Double, samplec As UInteger) As Integer
        End Function

        Public Shared Function ad_get_next_run_f64(adh As Integer, ByRef state As ad_scan_state, ByRef run As UInteger, ByRef samplev() As Double) As Integer
            Return ad_get_next_run_f64(adh, state, run, samplev, samplev.Length)
        End Function

        ' Current scan position
        <StructLayout(LayoutKind.Sequential, Pack:=0)>
        Public Structure ad_scan_pos
            Public run As UInteger
            Public offset As UInteger
        End Structure

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl)>
        Public Shared Function ad_get_trigger_pos(adh As Integer, ByRef pos As ad_scan_pos) As Integer
        End Function

        <DllImport("kernel32.dll")>
        Private Shared Function GetLastError() As UInteger
        End Function

        Public Shared ReadOnly Property errno() As Integer
            Get
                Return GetLastError()
            End Get
        End Property

        <StructLayout(LayoutKind.Sequential, Pack:=0)>
        Public Structure ad_counter_mode
            Public cha As Integer           ' counter channel
            Public mode As Byte             ' counter mode
            Public mux_a As Byte            ' A input mux setting
            Public mux_b As Byte            ' B input mux setting
            Public mux_rst As Byte          ' reset input mux setting

            Public flags As UShort          ' control flags

            Public capt_a As UInteger       ' capture registers
            Public capt_b As UInteger

            Public eact_a As Byte           ' event action on capt_a match
            Public eact_b As Byte           ' event action on capt_b match
        End Structure

        Private Const AD_IOCTL_SET_COUNTER_MODE As UInteger = &HB3800001UI
        Private Const AD_IOCTL_GET_COUNTER_MODE As UInteger = &HB3800002UI

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ad_ioctl")>
        Private Shared Function ad_ioctl_counter_mode(adh As Integer, ioc As UInteger, ByRef par As ad_counter_mode, size As Integer) As Integer
        End Function

        Public Shared Function ad_set_counter_mode(adh As Integer, ByRef mode As ad_counter_mode) As Integer
            ad_set_counter_mode = ad_ioctl_counter_mode(adh, AD_IOCTL_SET_COUNTER_MODE, mode, Marshal.SizeOf(mode))
        End Function

        Public Shared Function ad_get_counter_mode(adh As Integer, ByRef mode As ad_counter_mode) As Integer
            ad_get_counter_mode = ad_ioctl_counter_mode(adh, AD_IOCTL_GET_COUNTER_MODE, mode, Marshal.SizeOf(mode))
        End Function


        Public Const AD_CNT_COUNTER As Byte = 0          ' counter mode
        Public Const AD_CNT_UPDOWN As Byte = 1          ' up/down counter mode
        Public Const AD_CNT_QUAD_DECODER As Byte = 4          ' quadrature decoder
        Public Const AD_CNT_PULSE_TIME As Byte = 5

        Public Const AD_CNT_INV_A As UShort = &H1   ' invert A input
        Public Const AD_CNT_INV_B As UShort = &H2   ' invert B input
        Public Const AD_CNT_INV_RST As UShort = &H4   ' invert reset input
        Public Const AD_CNT_ENABLE_RST As UShort = &H8   ' enable reset input

        <StructLayout(LayoutKind.Sequential, Pack:=0)>
        Public Structure ad_product_info
            Public serial As UInteger       ' serial number
            Public fw_version As UInteger   ' firmware version

            <MarshalAsAttribute(UnmanagedType.ByValTStr, SizeConst:=32)>
            Public model As String          ' model name

            <MarshalAsAttribute(UnmanagedType.ByValArray, SizeConst:=256)>
            Public res() As Byte

        End Structure

        <DllImport("libad4.dll", CallingConvention:=CallingConvention.Cdecl, EntryPoint:="ad_get_product_info")>
        Private Shared Function ad_read_product_info(adh As Integer, id As Integer, ByRef info As ad_product_info, size As Integer) As Integer
        End Function


        Public Shared Function ad_get_product_info(adh As Integer, id As Integer, ByRef info As ad_product_info) As Integer
            ad_get_product_info = ad_read_product_info(adh, id, info, Marshal.SizeOf(info))
        End Function


        Public Shared Function AD_MAJOR_VERS(vers As UInteger) As Byte
            AD_MAJOR_VERS = (vers >> 24) And &HFF
        End Function

        Public Shared Function AD_MINOR_VERS(vers As UInteger) As Byte
            AD_MINOR_VERS = (vers >> 16) And &HFF
        End Function

        Public Shared Function AD_BUILD_VERS(vers As UInteger) As UShort
            AD_BUILD_VERS = (vers) And &HFFFF
        End Function

    End Class
End Namespace


